package javax.swing.plaf.basic;

import java.awt.event.MouseMotionListener;
import java.awt.Insets;
import javax.swing.SwingUtilities;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import javax.swing.Icon;
import javax.swing.ButtonModel;
import javax.swing.text.View;
import sun.swing.SwingUtilities2;
import java.awt.Graphics;
import javax.swing.plaf.UIResource;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.AbstractButton;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Rectangle;
import javax.swing.plaf.ButtonUI;

public class BasicButtonUI extends ButtonUI
{
    protected int defaultTextIconGap;
    private int shiftOffset;
    protected int defaultTextShiftOffset;
    private static final String propertyPrefix = "Button.";
    private static final Object BASIC_BUTTON_UI_KEY;
    private static Rectangle viewRect;
    private static Rectangle textRect;
    private static Rectangle iconRect;
    
    public BasicButtonUI() {
        this.shiftOffset = 0;
    }
    
    public static ComponentUI createUI(JComponent component) {
        AppContext appContext = AppContext.getAppContext();
        BasicButtonUI value = (BasicButtonUI)appContext.get(BasicButtonUI.BASIC_BUTTON_UI_KEY);
        if (value == null) {
            value = new BasicButtonUI();
            appContext.put(BasicButtonUI.BASIC_BUTTON_UI_KEY, value);
            return value;
        }
        return value;
    }
    
    protected String getPropertyPrefix() {
        return "Button.";
    }
    
    @Override
    public void installUI(JComponent component) {
        this.installDefaults((AbstractButton)component);
        this.installListeners((AbstractButton)component);
        this.installKeyboardActions((AbstractButton)component);
        BasicHTML.updateRenderer(component, ((AbstractButton)component).getText());
    }
    
    protected void installDefaults(AbstractButton abstractButton) {
        String propertyPrefix = this.getPropertyPrefix();
        this.defaultTextShiftOffset = UIManager.getInt(propertyPrefix + "textShiftOffset");
        if (abstractButton.isContentAreaFilled()) {
            LookAndFeel.installProperty(abstractButton, "opaque", Boolean.TRUE);
        }
        else {
            LookAndFeel.installProperty(abstractButton, "opaque", Boolean.FALSE);
        }
        if (abstractButton.getMargin() == null || abstractButton.getMargin() instanceof UIResource) {
            abstractButton.setMargin(UIManager.getInsets(propertyPrefix + "margin"));
        }
        LookAndFeel.installColorsAndFont(abstractButton, propertyPrefix + "background", propertyPrefix + "foreground", propertyPrefix + "font");
        LookAndFeel.installBorder(abstractButton, propertyPrefix + "border");
        Object value = UIManager.get(propertyPrefix + "rollover");
        if (value != null) {
            LookAndFeel.installProperty(abstractButton, "rolloverEnabled", value);
        }
        LookAndFeel.installProperty(abstractButton, "iconTextGap", Integer.valueOf(4));
    }
    
    protected void installListeners(AbstractButton abstractButton) {
        BasicButtonListener buttonListener = this.createButtonListener(abstractButton);
        if (buttonListener != null) {
            abstractButton.addMouseListener(buttonListener);
            abstractButton.addMouseMotionListener(buttonListener);
            abstractButton.addFocusListener(buttonListener);
            abstractButton.addPropertyChangeListener(buttonListener);
            abstractButton.addChangeListener(buttonListener);
        }
    }
    
    protected void installKeyboardActions(AbstractButton abstractButton) {
        BasicButtonListener buttonListener = this.getButtonListener(abstractButton);
        if (buttonListener != null) {
            buttonListener.installKeyboardActions(abstractButton);
        }
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallKeyboardActions((AbstractButton)component);
        this.uninstallListeners((AbstractButton)component);
        this.uninstallDefaults((AbstractButton)component);
        BasicHTML.updateRenderer(component, "");
    }
    
    protected void uninstallKeyboardActions(AbstractButton abstractButton) {
        BasicButtonListener buttonListener = this.getButtonListener(abstractButton);
        if (buttonListener != null) {
            buttonListener.uninstallKeyboardActions(abstractButton);
        }
    }
    
    protected void uninstallListeners(AbstractButton abstractButton) {
        BasicButtonListener buttonListener = this.getButtonListener(abstractButton);
        if (buttonListener != null) {
            abstractButton.removeMouseListener(buttonListener);
            abstractButton.removeMouseMotionListener(buttonListener);
            abstractButton.removeFocusListener(buttonListener);
            abstractButton.removeChangeListener(buttonListener);
            abstractButton.removePropertyChangeListener(buttonListener);
        }
    }
    
    protected void uninstallDefaults(AbstractButton c) {
        LookAndFeel.uninstallBorder(c);
    }
    
    protected BasicButtonListener createButtonListener(AbstractButton abstractButton) {
        return new BasicButtonListener(abstractButton);
    }
    
    public int getDefaultTextIconGap(AbstractButton abstractButton) {
        return this.defaultTextIconGap;
    }
    
    @Override
    public void paint(Graphics g, JComponent component) {
        AbstractButton c = (AbstractButton)component;
        ButtonModel model = c.getModel();
        String layout = this.layout(c, SwingUtilities2.getFontMetrics(c, g), c.getWidth(), c.getHeight());
        this.clearTextShiftOffset();
        if (model.isArmed() && model.isPressed()) {
            this.paintButtonPressed(g, c);
        }
        if (c.getIcon() != null) {
            this.paintIcon(g, component, BasicButtonUI.iconRect);
        }
        if (layout != null && !layout.equals("")) {
            View view = (View)component.getClientProperty("html");
            if (view != null) {
                view.paint(g, BasicButtonUI.textRect);
            }
            else {
                this.paintText(g, c, BasicButtonUI.textRect, layout);
            }
        }
        if (c.isFocusPainted() && c.hasFocus()) {
            this.paintFocus(g, c, BasicButtonUI.viewRect, BasicButtonUI.textRect, BasicButtonUI.iconRect);
        }
    }
    
    protected void paintIcon(Graphics graphics, JComponent component, Rectangle rectangle) {
        AbstractButton abstractButton = (AbstractButton)component;
        ButtonModel model = abstractButton.getModel();
        Icon icon = abstractButton.getIcon();
        Icon icon2 = null;
        if (icon == null) {
            return;
        }
        Icon selectedIcon = null;
        if (model.isSelected()) {
            selectedIcon = abstractButton.getSelectedIcon();
            if (selectedIcon != null) {
                icon = selectedIcon;
            }
        }
        if (!model.isEnabled()) {
            if (model.isSelected()) {
                icon2 = abstractButton.getDisabledSelectedIcon();
                if (icon2 == null) {
                    icon2 = selectedIcon;
                }
            }
            if (icon2 == null) {
                icon2 = abstractButton.getDisabledIcon();
            }
        }
        else if (model.isPressed() && model.isArmed()) {
            icon2 = abstractButton.getPressedIcon();
            if (icon2 != null) {
                this.clearTextShiftOffset();
            }
        }
        else if (abstractButton.isRolloverEnabled() && model.isRollover()) {
            if (model.isSelected()) {
                icon2 = abstractButton.getRolloverSelectedIcon();
                if (icon2 == null) {
                    icon2 = selectedIcon;
                }
            }
            if (icon2 == null) {
                icon2 = abstractButton.getRolloverIcon();
            }
        }
        if (icon2 != null) {
            icon = icon2;
        }
        if (model.isPressed() && model.isArmed()) {
            icon.paintIcon(component, graphics, rectangle.x + this.getTextShiftOffset(), rectangle.y + this.getTextShiftOffset());
        }
        else {
            icon.paintIcon(component, graphics, rectangle.x, rectangle.y);
        }
        return;
    }
    
    protected void paintText(Graphics graphics, JComponent component, Rectangle rectangle, String text) {
        AbstractButton abstractButton = (AbstractButton)component;
        ButtonModel model = abstractButton.getModel();
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(component, graphics);
        int displayedMnemonicIndex = abstractButton.getDisplayedMnemonicIndex();
        if (model.isEnabled()) {
            graphics.setColor(abstractButton.getForeground());
            SwingUtilities2.drawStringUnderlineCharAt(component, graphics, text, displayedMnemonicIndex, rectangle.x + this.getTextShiftOffset(), rectangle.y + fontMetrics.getAscent() + this.getTextShiftOffset());
        }
        else {
            graphics.setColor(abstractButton.getBackground().brighter());
            SwingUtilities2.drawStringUnderlineCharAt(component, graphics, text, displayedMnemonicIndex, rectangle.x, rectangle.y + fontMetrics.getAscent());
            graphics.setColor(abstractButton.getBackground().darker());
            SwingUtilities2.drawStringUnderlineCharAt(component, graphics, text, displayedMnemonicIndex, rectangle.x - 1, rectangle.y + fontMetrics.getAscent() - 1);
        }
    }
    
    protected void paintText(Graphics graphics, AbstractButton abstractButton, Rectangle rectangle, String s) {
        this.paintText(graphics, abstractButton, rectangle, s);
    }
    
    protected void paintFocus(Graphics graphics, AbstractButton abstractButton, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3) {
    }
    
    protected void paintButtonPressed(Graphics graphics, AbstractButton abstractButton) {
    }
    
    protected void clearTextShiftOffset() {
        this.shiftOffset = 0;
    }
    
    protected void setTextShiftOffset() {
        this.shiftOffset = this.defaultTextShiftOffset;
    }
    
    protected int getTextShiftOffset() {
        return this.shiftOffset;
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width - (view.getPreferredSpan(0) - view.getMinimumSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        AbstractButton abstractButton = (AbstractButton)component;
        return BasicGraphicsUtils.getPreferredButtonSize(abstractButton, abstractButton.getIconTextGap());
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width + (view.getMaximumSpan(0) - view.getPreferredSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        AbstractButton abstractButton = (AbstractButton)c;
        String text = abstractButton.getText();
        if (text == null || "".equals(text)) {
            return -1;
        }
        FontMetrics fontMetrics = abstractButton.getFontMetrics(abstractButton.getFont());
        this.layout(abstractButton, fontMetrics, width, height);
        return BasicHTML.getBaseline(abstractButton, BasicButtonUI.textRect.y, fontMetrics.getAscent(), BasicButtonUI.textRect.width, BasicButtonUI.textRect.height);
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        if (c.getClientProperty("html") != null) {
            return Component.BaselineResizeBehavior.OTHER;
        }
        switch (((AbstractButton)c).getVerticalAlignment()) {
            case 1: {
                return Component.BaselineResizeBehavior.CONSTANT_ASCENT;
            }
            case 3: {
                return Component.BaselineResizeBehavior.CONSTANT_DESCENT;
            }
            case 0: {
                return Component.BaselineResizeBehavior.CENTER_OFFSET;
            }
            default: {
                return Component.BaselineResizeBehavior.OTHER;
            }
        }
    }
    
    private String layout(AbstractButton c, FontMetrics fm, int n, int n2) {
        Insets insets = c.getInsets();
        BasicButtonUI.viewRect.x = insets.left;
        BasicButtonUI.viewRect.y = insets.top;
        BasicButtonUI.viewRect.width = n - (insets.right + BasicButtonUI.viewRect.x);
        BasicButtonUI.viewRect.height = n2 - (insets.bottom + BasicButtonUI.viewRect.y);
        Rectangle textRect = BasicButtonUI.textRect;
        Rectangle textRect2 = BasicButtonUI.textRect;
        Rectangle textRect3 = BasicButtonUI.textRect;
        Rectangle textRect4 = BasicButtonUI.textRect;
        int n3 = 0;
        textRect4.height = n3;
        textRect3.width = n3;
        textRect2.y = n3;
        textRect.x = n3;
        Rectangle iconRect = BasicButtonUI.iconRect;
        Rectangle iconRect2 = BasicButtonUI.iconRect;
        Rectangle iconRect3 = BasicButtonUI.iconRect;
        Rectangle iconRect4 = BasicButtonUI.iconRect;
        int n4 = 0;
        iconRect4.height = n4;
        iconRect3.width = n4;
        iconRect2.y = n4;
        iconRect.x = n4;
        return SwingUtilities.layoutCompoundLabel(c, fm, c.getText(), c.getIcon(), c.getVerticalAlignment(), c.getHorizontalAlignment(), c.getVerticalTextPosition(), c.getHorizontalTextPosition(), BasicButtonUI.viewRect, BasicButtonUI.iconRect, BasicButtonUI.textRect, (c.getText() == null) ? 0 : c.getIconTextGap());
    }
    
    private BasicButtonListener getButtonListener(AbstractButton abstractButton) {
        MouseMotionListener[] mouseMotionListeners = abstractButton.getMouseMotionListeners();
        if (mouseMotionListeners != null) {
            int length = mouseMotionListeners.length;
            int i = 0;
            while (i < length) {
                MouseMotionListener mouseMotionListener = mouseMotionListeners[i];
                if ((mouseMotionListener instanceof BasicButtonListener)) {
                    return (BasicButtonListener)mouseMotionListener;
                }
                ++i;
            }
        }
        return null;
    }
    
    static {
        BASIC_BUTTON_UI_KEY = new Object();
        BasicButtonUI.viewRect = new Rectangle();
        BasicButtonUI.textRect = new Rectangle();
        BasicButtonUI.iconRect = new Rectangle();
    }
}
